/* 
 *  Copyright (C) 2004 Christian Loose <christian.loose@kdemail.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "dirignorelist.h"
using namespace Cervisia;

#include <qfileinfo.h>


DirIgnoreList::DirIgnoreList(const QString& path)
{
    addEntriesFromFile(path + "/.cvsignore");
}


void DirIgnoreList::addEntry(const QString& entry)
{
    if (entry != QLatin1String("!"))
    {
        m_stringMatcher.add(entry);
    }
    else
    {
        m_stringMatcher.clear();
    }
}


bool DirIgnoreList::matches(const QFileInfo* fi) const
{  
    return m_stringMatcher.match(fi->fileName());
}
